# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: injective/exchange/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos.distribution.v1beta1 import distribution_pb2 as cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2
from cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from injective.exchange.v1beta1 import exchange_pb2 as injective_dot_exchange_dot_v1beta1_dot_exchange__pb2
from injective.oracle.v1beta1 import oracle_pb2 as injective_dot_oracle_dot_v1beta1_dot_oracle__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#injective/exchange/v1beta1/tx.proto\x12\x1ainjective.exchange.v1beta1\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a.cosmos/distribution/v1beta1/distribution.proto\x1a\x17\x63osmos/msg/v1/msg.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x14gogoproto/gogo.proto\x1a)injective/exchange/v1beta1/exchange.proto\x1a%injective/oracle/v1beta1/oracle.proto\"\x88\x01\n\x0fMsgUpdateParams\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x38\n\x06params\x18\x02 \x01(\x0b\x32\".injective.exchange.v1beta1.ParamsB\x04\xc8\xde\x1f\x00:\x0e\x82\xe7\xb0*\tauthority\"\x19\n\x17MsgUpdateParamsResponse\"y\n\nMsgDeposit\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\t\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x14\n\x12MsgDepositResponse\"z\n\x0bMsgWithdraw\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\t\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x15\n\x13MsgWithdrawResponse\"z\n\x17MsgCreateSpotLimitOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12:\n\x05order\x18\x02 \x01(\x0b\x32%.injective.exchange.v1beta1.SpotOrderB\x04\xc8\xde\x1f\x00:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"?\n\x1fMsgCreateSpotLimitOrderResponse\x12\x12\n\norder_hash\x18\x01 \x01(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x81\x01\n\x1dMsgBatchCreateSpotLimitOrders\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12;\n\x06orders\x18\x02 \x03(\x0b\x32%.injective.exchange.v1beta1.SpotOrderB\x04\xc8\xde\x1f\x00:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"G\n%MsgBatchCreateSpotLimitOrdersResponse\x12\x14\n\x0corder_hashes\x18\x01 \x03(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x97\x02\n\x1aMsgInstantSpotMarketLaunch\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x0e\n\x06ticker\x18\x02 \x01(\t\x12\x12\n\nbase_denom\x18\x03 \x01(\t\x12\x13\n\x0bquote_denom\x18\x04 \x01(\t\x12K\n\x13min_price_tick_size\x18\x05 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12N\n\x16min_quantity_tick_size\x18\x06 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"$\n\"MsgInstantSpotMarketLaunchResponse\"\xbb\x05\n\x1fMsgInstantPerpetualMarketLaunch\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x0e\n\x06ticker\x18\x02 \x01(\t\x12\x13\n\x0bquote_denom\x18\x03 \x01(\t\x12\x13\n\x0boracle_base\x18\x04 \x01(\t\x12\x14\n\x0coracle_quote\x18\x05 \x01(\t\x12\x1b\n\x13oracle_scale_factor\x18\x06 \x01(\r\x12\x39\n\x0boracle_type\x18\x07 \x01(\x0e\x32$.injective.oracle.v1beta1.OracleType\x12\x46\n\x0emaker_fee_rate\x18\x08 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12\x46\n\x0etaker_fee_rate\x18\t \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12L\n\x14initial_margin_ratio\x18\n \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12P\n\x18maintenance_margin_ratio\x18\x0b \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12K\n\x13min_price_tick_size\x18\x0c \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12N\n\x16min_quantity_tick_size\x18\r \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\")\n\'MsgInstantPerpetualMarketLaunchResponse\"\xef\x04\n#MsgInstantBinaryOptionsMarketLaunch\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x0e\n\x06ticker\x18\x02 \x01(\t\x12\x15\n\roracle_symbol\x18\x03 \x01(\t\x12\x17\n\x0foracle_provider\x18\x04 \x01(\t\x12\x39\n\x0boracle_type\x18\x05 \x01(\x0e\x32$.injective.oracle.v1beta1.OracleType\x12\x1b\n\x13oracle_scale_factor\x18\x06 \x01(\r\x12\x46\n\x0emaker_fee_rate\x18\x07 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12\x46\n\x0etaker_fee_rate\x18\x08 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12\x1c\n\x14\x65xpiration_timestamp\x18\t \x01(\x03\x12\x1c\n\x14settlement_timestamp\x18\n \x01(\x03\x12\r\n\x05\x61\x64min\x18\x0b \x01(\t\x12\x13\n\x0bquote_denom\x18\x0c \x01(\t\x12K\n\x13min_price_tick_size\x18\r \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12N\n\x16min_quantity_tick_size\x18\x0e \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"-\n+MsgInstantBinaryOptionsMarketLaunchResponse\"\xcf\x05\n#MsgInstantExpiryFuturesMarketLaunch\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x0e\n\x06ticker\x18\x02 \x01(\t\x12\x13\n\x0bquote_denom\x18\x03 \x01(\t\x12\x13\n\x0boracle_base\x18\x04 \x01(\t\x12\x14\n\x0coracle_quote\x18\x05 \x01(\t\x12\x39\n\x0boracle_type\x18\x06 \x01(\x0e\x32$.injective.oracle.v1beta1.OracleType\x12\x1b\n\x13oracle_scale_factor\x18\x07 \x01(\r\x12\x0e\n\x06\x65xpiry\x18\x08 \x01(\x03\x12\x46\n\x0emaker_fee_rate\x18\t \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12\x46\n\x0etaker_fee_rate\x18\n \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12L\n\x14initial_margin_ratio\x18\x0b \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12P\n\x18maintenance_margin_ratio\x18\x0c \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12K\n\x13min_price_tick_size\x18\r \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12N\n\x16min_quantity_tick_size\x18\x0e \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"-\n+MsgInstantExpiryFuturesMarketLaunchResponse\"{\n\x18MsgCreateSpotMarketOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12:\n\x05order\x18\x02 \x01(\x0b\x32%.injective.exchange.v1beta1.SpotOrderB\x04\xc8\xde\x1f\x00:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x8b\x01\n MsgCreateSpotMarketOrderResponse\x12\x12\n\norder_hash\x18\x01 \x01(\t\x12I\n\x07results\x18\x02 \x01(\x0b\x32\x32.injective.exchange.v1beta1.SpotMarketOrderResultsB\x04\xc8\xde\x1f\x01:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\xe0\x01\n\x16SpotMarketOrderResults\x12@\n\x08quantity\x18\x01 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12=\n\x05price\x18\x02 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12;\n\x03\x66\x65\x65\x18\x03 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x82\x01\n\x1dMsgCreateDerivativeLimitOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12@\n\x05order\x18\x02 \x01(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"E\n%MsgCreateDerivativeLimitOrderResponse\x12\x12\n\norder_hash\x18\x01 \x01(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x85\x01\n MsgCreateBinaryOptionsLimitOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12@\n\x05order\x18\x02 \x01(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"H\n(MsgCreateBinaryOptionsLimitOrderResponse\x12\x12\n\norder_hash\x18\x01 \x01(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x89\x01\n#MsgBatchCreateDerivativeLimitOrders\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x41\n\x06orders\x18\x02 \x03(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"M\n+MsgBatchCreateDerivativeLimitOrdersResponse\x12\x14\n\x0corder_hashes\x18\x01 \x03(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x80\x01\n\x12MsgCancelSpotOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x11\n\tmarket_id\x18\x02 \x01(\t\x12\x15\n\rsubaccount_id\x18\x03 \x01(\t\x12\x12\n\norder_hash\x18\x04 \x01(\t\x12\x0b\n\x03\x63id\x18\x05 \x01(\t:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x1c\n\x1aMsgCancelSpotOrderResponse\"v\n\x18MsgBatchCancelSpotOrders\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x39\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32%.injective.exchange.v1beta1.OrderDataB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"=\n MsgBatchCancelSpotOrdersResponse\x12\x0f\n\x07success\x18\x01 \x03(\x08:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x7f\n!MsgBatchCancelBinaryOptionsOrders\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x39\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32%.injective.exchange.v1beta1.OrderDataB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"F\n)MsgBatchCancelBinaryOptionsOrdersResponse\x12\x0f\n\x07success\x18\x01 \x03(\x08:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\xc7\x05\n\x14MsgBatchUpdateOrders\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\t\x12%\n\x1dspot_market_ids_to_cancel_all\x18\x03 \x03(\t\x12+\n#derivative_market_ids_to_cancel_all\x18\x04 \x03(\t\x12J\n\x15spot_orders_to_cancel\x18\x05 \x03(\x0b\x32%.injective.exchange.v1beta1.OrderDataB\x04\xc8\xde\x1f\x01\x12P\n\x1b\x64\x65rivative_orders_to_cancel\x18\x06 \x03(\x0b\x32%.injective.exchange.v1beta1.OrderDataB\x04\xc8\xde\x1f\x01\x12J\n\x15spot_orders_to_create\x18\x07 \x03(\x0b\x32%.injective.exchange.v1beta1.SpotOrderB\x04\xc8\xde\x1f\x01\x12V\n\x1b\x64\x65rivative_orders_to_create\x18\x08 \x03(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x01\x12T\n\x1f\x62inary_options_orders_to_cancel\x18\t \x03(\x0b\x32%.injective.exchange.v1beta1.OrderDataB\x04\xc8\xde\x1f\x01\x12/\n\'binary_options_market_ids_to_cancel_all\x18\n \x03(\t\x12Z\n\x1f\x62inary_options_orders_to_create\x18\x0b \x03(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x01:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\xf0\x01\n\x1cMsgBatchUpdateOrdersResponse\x12\x1b\n\x13spot_cancel_success\x18\x01 \x03(\x08\x12!\n\x19\x64\x65rivative_cancel_success\x18\x02 \x03(\x08\x12\x19\n\x11spot_order_hashes\x18\x03 \x03(\t\x12\x1f\n\x17\x64\x65rivative_order_hashes\x18\x04 \x03(\t\x12%\n\x1d\x62inary_options_cancel_success\x18\x05 \x03(\x08\x12#\n\x1b\x62inary_options_order_hashes\x18\x06 \x03(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x83\x01\n\x1eMsgCreateDerivativeMarketOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12@\n\x05order\x18\x02 \x01(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x97\x01\n&MsgCreateDerivativeMarketOrderResponse\x12\x12\n\norder_hash\x18\x01 \x01(\t\x12O\n\x07results\x18\x02 \x01(\x0b\x32\x38.injective.exchange.v1beta1.DerivativeMarketOrderResultsB\x04\xc8\xde\x1f\x01:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\xef\x02\n\x1c\x44\x65rivativeMarketOrderResults\x12@\n\x08quantity\x18\x01 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12=\n\x05price\x18\x02 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12;\n\x03\x66\x65\x65\x18\x03 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12G\n\x0eposition_delta\x18\x04 \x01(\x0b\x32).injective.exchange.v1beta1.PositionDeltaB\x04\xc8\xde\x1f\x00\x12>\n\x06payout\x18\x05 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x86\x01\n!MsgCreateBinaryOptionsMarketOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12@\n\x05order\x18\x02 \x01(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x9a\x01\n)MsgCreateBinaryOptionsMarketOrderResponse\x12\x12\n\norder_hash\x18\x01 \x01(\t\x12O\n\x07results\x18\x02 \x01(\x0b\x32\x38.injective.exchange.v1beta1.DerivativeMarketOrderResultsB\x04\xc8\xde\x1f\x01:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\x9a\x01\n\x18MsgCancelDerivativeOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x11\n\tmarket_id\x18\x02 \x01(\t\x12\x15\n\rsubaccount_id\x18\x03 \x01(\t\x12\x12\n\norder_hash\x18\x04 \x01(\t\x12\x12\n\norder_mask\x18\x05 \x01(\x05\x12\x0b\n\x03\x63id\x18\x06 \x01(\t:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\"\n MsgCancelDerivativeOrderResponse\"\x9d\x01\n\x1bMsgCancelBinaryOptionsOrder\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x11\n\tmarket_id\x18\x02 \x01(\t\x12\x15\n\rsubaccount_id\x18\x03 \x01(\t\x12\x12\n\norder_hash\x18\x04 \x01(\t\x12\x12\n\norder_mask\x18\x05 \x01(\x05\x12\x0b\n\x03\x63id\x18\x06 \x01(\t:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"%\n#MsgCancelBinaryOptionsOrderResponse\"j\n\tOrderData\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\t\x12\x12\n\norder_hash\x18\x03 \x01(\t\x12\x12\n\norder_mask\x18\x04 \x01(\x05\x12\x0b\n\x03\x63id\x18\x05 \x01(\t\"|\n\x1eMsgBatchCancelDerivativeOrders\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x39\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32%.injective.exchange.v1beta1.OrderDataB\x04\xc8\xde\x1f\x00:\x0f\x88\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"C\n&MsgBatchCancelDerivativeOrdersResponse\x12\x0f\n\x07success\x18\x01 \x03(\x08:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"\xa6\x01\n\x15MsgSubaccountTransfer\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x1c\n\x14source_subaccount_id\x18\x02 \x01(\t\x12!\n\x19\x64\x65stination_subaccount_id\x18\x03 \x01(\t\x12/\n\x06\x61mount\x18\x04 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00:\x0b\x82\xe7\xb0*\x06sender\"\x1f\n\x1dMsgSubaccountTransferResponse\"\xa4\x01\n\x13MsgExternalTransfer\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x1c\n\x14source_subaccount_id\x18\x02 \x01(\t\x12!\n\x19\x64\x65stination_subaccount_id\x18\x03 \x01(\t\x12/\n\x06\x61mount\x18\x04 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00:\x0b\x82\xe7\xb0*\x06sender\"\x1d\n\x1bMsgExternalTransferResponse\"\x9f\x01\n\x14MsgLiquidatePosition\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\t\x12\x11\n\tmarket_id\x18\x03 \x01(\t\x12@\n\x05order\x18\x04 \x01(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrderB\x04\xc8\xde\x1f\x01:\x0b\x82\xe7\xb0*\x06sender\"\x1e\n\x1cMsgLiquidatePositionResponse\"\xcc\x01\n\x19MsgIncreasePositionMargin\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x1c\n\x14source_subaccount_id\x18\x02 \x01(\t\x12!\n\x19\x64\x65stination_subaccount_id\x18\x03 \x01(\t\x12\x11\n\tmarket_id\x18\x04 \x01(\t\x12>\n\x06\x61mount\x18\x05 \x01(\tB.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec:\x0b\x82\xe7\xb0*\x06sender\"#\n!MsgIncreasePositionMarginResponse\"z\n\x1cMsgPrivilegedExecuteContract\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\r\n\x05\x66unds\x18\x02 \x01(\t\x12\x18\n\x10\x63ontract_address\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\x9c\x01\n$MsgPrivilegedExecuteContractResponse\x12_\n\nfunds_diff\x18\x01 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins:\x13\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\x82\xe7\xb0*\x06sender\"\"\n\x10MsgRewardsOptOut\x12\x0e\n\x06sender\x18\x01 \x01(\t\"\x1a\n\x18MsgRewardsOptOutResponse\"d\n\x15MsgReclaimLockedFunds\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x1b\n\x13lockedAccountPubKey\x18\x02 \x01(\x0c\x12\x11\n\tsignature\x18\x03 \x01(\x0c:\x0b\x82\xe7\xb0*\x06sender\"\x1f\n\x1dMsgReclaimLockedFundsResponse\"r\n\x0bMsgSignData\x12K\n\x06Signer\x18\x01 \x01(\x0c\x42;\xea\xde\x1f\x06signer\xfa\xde\x1f-github.com/cosmos/cosmos-sdk/types.AccAddress\x12\x16\n\x04\x44\x61ta\x18\x02 \x01(\x0c\x42\x08\xea\xde\x1f\x04\x64\x61ta\"g\n\nMsgSignDoc\x12\x1b\n\tsign_type\x18\x01 \x01(\tB\x08\xea\xde\x1f\x04type\x12<\n\x05value\x18\x02 \x01(\x0b\x32\'.injective.exchange.v1beta1.MsgSignDataB\x04\xc8\xde\x1f\x00\"\x93\x02\n!MsgAdminUpdateBinaryOptionsMarket\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x11\n\tmarket_id\x18\x02 \x01(\t\x12H\n\x10settlement_price\x18\x03 \x01(\tB.\xc8\xde\x1f\x01\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12\x1c\n\x14\x65xpiration_timestamp\x18\x04 \x01(\x03\x12\x1c\n\x14settlement_timestamp\x18\x05 \x01(\x03\x12\x38\n\x06status\x18\x06 \x01(\x0e\x32(.injective.exchange.v1beta1.MarketStatus:\x0b\x82\xe7\xb0*\x06sender\"+\n)MsgAdminUpdateBinaryOptionsMarketResponse2\x90\"\n\x03Msg\x12\x61\n\x07\x44\x65posit\x12&.injective.exchange.v1beta1.MsgDeposit\x1a..injective.exchange.v1beta1.MsgDepositResponse\x12\x64\n\x08Withdraw\x12\'.injective.exchange.v1beta1.MsgWithdraw\x1a/.injective.exchange.v1beta1.MsgWithdrawResponse\x12\x91\x01\n\x17InstantSpotMarketLaunch\x12\x36.injective.exchange.v1beta1.MsgInstantSpotMarketLaunch\x1a>.injective.exchange.v1beta1.MsgInstantSpotMarketLaunchResponse\x12\xa0\x01\n\x1cInstantPerpetualMarketLaunch\x12;.injective.exchange.v1beta1.MsgInstantPerpetualMarketLaunch\x1a\x43.injective.exchange.v1beta1.MsgInstantPerpetualMarketLaunchResponse\x12\xac\x01\n InstantExpiryFuturesMarketLaunch\x12?.injective.exchange.v1beta1.MsgInstantExpiryFuturesMarketLaunch\x1aG.injective.exchange.v1beta1.MsgInstantExpiryFuturesMarketLaunchResponse\x12\x88\x01\n\x14\x43reateSpotLimitOrder\x12\x33.injective.exchange.v1beta1.MsgCreateSpotLimitOrder\x1a;.injective.exchange.v1beta1.MsgCreateSpotLimitOrderResponse\x12\x9a\x01\n\x1a\x42\x61tchCreateSpotLimitOrders\x12\x39.injective.exchange.v1beta1.MsgBatchCreateSpotLimitOrders\x1a\x41.injective.exchange.v1beta1.MsgBatchCreateSpotLimitOrdersResponse\x12\x8b\x01\n\x15\x43reateSpotMarketOrder\x12\x34.injective.exchange.v1beta1.MsgCreateSpotMarketOrder\x1a<.injective.exchange.v1beta1.MsgCreateSpotMarketOrderResponse\x12y\n\x0f\x43\x61ncelSpotOrder\x12..injective.exchange.v1beta1.MsgCancelSpotOrder\x1a\x36.injective.exchange.v1beta1.MsgCancelSpotOrderResponse\x12\x8b\x01\n\x15\x42\x61tchCancelSpotOrders\x12\x34.injective.exchange.v1beta1.MsgBatchCancelSpotOrders\x1a<.injective.exchange.v1beta1.MsgBatchCancelSpotOrdersResponse\x12\x7f\n\x11\x42\x61tchUpdateOrders\x12\x30.injective.exchange.v1beta1.MsgBatchUpdateOrders\x1a\x38.injective.exchange.v1beta1.MsgBatchUpdateOrdersResponse\x12\x97\x01\n\x19PrivilegedExecuteContract\x12\x38.injective.exchange.v1beta1.MsgPrivilegedExecuteContract\x1a@.injective.exchange.v1beta1.MsgPrivilegedExecuteContractResponse\x12\x9a\x01\n\x1a\x43reateDerivativeLimitOrder\x12\x39.injective.exchange.v1beta1.MsgCreateDerivativeLimitOrder\x1a\x41.injective.exchange.v1beta1.MsgCreateDerivativeLimitOrderResponse\x12\xac\x01\n BatchCreateDerivativeLimitOrders\x12?.injective.exchange.v1beta1.MsgBatchCreateDerivativeLimitOrders\x1aG.injective.exchange.v1beta1.MsgBatchCreateDerivativeLimitOrdersResponse\x12\x9d\x01\n\x1b\x43reateDerivativeMarketOrder\x12:.injective.exchange.v1beta1.MsgCreateDerivativeMarketOrder\x1a\x42.injective.exchange.v1beta1.MsgCreateDerivativeMarketOrderResponse\x12\x8b\x01\n\x15\x43\x61ncelDerivativeOrder\x12\x34.injective.exchange.v1beta1.MsgCancelDerivativeOrder\x1a<.injective.exchange.v1beta1.MsgCancelDerivativeOrderResponse\x12\x9d\x01\n\x1b\x42\x61tchCancelDerivativeOrders\x12:.injective.exchange.v1beta1.MsgBatchCancelDerivativeOrders\x1a\x42.injective.exchange.v1beta1.MsgBatchCancelDerivativeOrdersResponse\x12\xac\x01\n InstantBinaryOptionsMarketLaunch\x12?.injective.exchange.v1beta1.MsgInstantBinaryOptionsMarketLaunch\x1aG.injective.exchange.v1beta1.MsgInstantBinaryOptionsMarketLaunchResponse\x12\xa3\x01\n\x1d\x43reateBinaryOptionsLimitOrder\x12<.injective.exchange.v1beta1.MsgCreateBinaryOptionsLimitOrder\x1a\x44.injective.exchange.v1beta1.MsgCreateBinaryOptionsLimitOrderResponse\x12\xa6\x01\n\x1e\x43reateBinaryOptionsMarketOrder\x12=.injective.exchange.v1beta1.MsgCreateBinaryOptionsMarketOrder\x1a\x45.injective.exchange.v1beta1.MsgCreateBinaryOptionsMarketOrderResponse\x12\x94\x01\n\x18\x43\x61ncelBinaryOptionsOrder\x12\x37.injective.exchange.v1beta1.MsgCancelBinaryOptionsOrder\x1a?.injective.exchange.v1beta1.MsgCancelBinaryOptionsOrderResponse\x12\xa6\x01\n\x1e\x42\x61tchCancelBinaryOptionsOrders\x12=.injective.exchange.v1beta1.MsgBatchCancelBinaryOptionsOrders\x1a\x45.injective.exchange.v1beta1.MsgBatchCancelBinaryOptionsOrdersResponse\x12\x82\x01\n\x12SubaccountTransfer\x12\x31.injective.exchange.v1beta1.MsgSubaccountTransfer\x1a\x39.injective.exchange.v1beta1.MsgSubaccountTransferResponse\x12|\n\x10\x45xternalTransfer\x12/.injective.exchange.v1beta1.MsgExternalTransfer\x1a\x37.injective.exchange.v1beta1.MsgExternalTransferResponse\x12\x7f\n\x11LiquidatePosition\x12\x30.injective.exchange.v1beta1.MsgLiquidatePosition\x1a\x38.injective.exchange.v1beta1.MsgLiquidatePositionResponse\x12\x8e\x01\n\x16IncreasePositionMargin\x12\x35.injective.exchange.v1beta1.MsgIncreasePositionMargin\x1a=.injective.exchange.v1beta1.MsgIncreasePositionMarginResponse\x12s\n\rRewardsOptOut\x12,.injective.exchange.v1beta1.MsgRewardsOptOut\x1a\x34.injective.exchange.v1beta1.MsgRewardsOptOutResponse\x12\xa6\x01\n\x1e\x41\x64minUpdateBinaryOptionsMarket\x12=.injective.exchange.v1beta1.MsgAdminUpdateBinaryOptionsMarket\x1a\x45.injective.exchange.v1beta1.MsgAdminUpdateBinaryOptionsMarketResponse\x12\x82\x01\n\x12ReclaimLockedFunds\x12\x31.injective.exchange.v1beta1.MsgReclaimLockedFunds\x1a\x39.injective.exchange.v1beta1.MsgReclaimLockedFundsResponse\x12p\n\x0cUpdateParams\x12+.injective.exchange.v1beta1.MsgUpdateParams\x1a\x33.injective.exchange.v1beta1.MsgUpdateParamsResponseBPZNgithub.com/InjectiveLabs/injective-core/injective-chain/modules/exchange/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'injective.exchange.v1beta1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZNgithub.com/InjectiveLabs/injective-core/injective-chain/modules/exchange/types'
  _MSGUPDATEPARAMS.fields_by_name['authority']._options = None
  _MSGUPDATEPARAMS.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEPARAMS.fields_by_name['params']._options = None
  _MSGUPDATEPARAMS.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEPARAMS._options = None
  _MSGUPDATEPARAMS._serialized_options = b'\202\347\260*\tauthority'
  _MSGDEPOSIT.fields_by_name['amount']._options = None
  _MSGDEPOSIT.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGDEPOSIT._options = None
  _MSGDEPOSIT._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGWITHDRAW.fields_by_name['amount']._options = None
  _MSGWITHDRAW.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGWITHDRAW._options = None
  _MSGWITHDRAW._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGCREATESPOTLIMITORDER.fields_by_name['order']._options = None
  _MSGCREATESPOTLIMITORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _MSGCREATESPOTLIMITORDER._options = None
  _MSGCREATESPOTLIMITORDER._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGCREATESPOTLIMITORDERRESPONSE._options = None
  _MSGCREATESPOTLIMITORDERRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGBATCHCREATESPOTLIMITORDERS.fields_by_name['orders']._options = None
  _MSGBATCHCREATESPOTLIMITORDERS.fields_by_name['orders']._serialized_options = b'\310\336\037\000'
  _MSGBATCHCREATESPOTLIMITORDERS._options = None
  _MSGBATCHCREATESPOTLIMITORDERS._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGBATCHCREATESPOTLIMITORDERSRESPONSE._options = None
  _MSGBATCHCREATESPOTLIMITORDERSRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGINSTANTSPOTMARKETLAUNCH.fields_by_name['min_price_tick_size']._options = None
  _MSGINSTANTSPOTMARKETLAUNCH.fields_by_name['min_price_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTSPOTMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._options = None
  _MSGINSTANTSPOTMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTSPOTMARKETLAUNCH._options = None
  _MSGINSTANTSPOTMARKETLAUNCH._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['maker_fee_rate']._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['maker_fee_rate']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['taker_fee_rate']._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['taker_fee_rate']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['initial_margin_ratio']._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['initial_margin_ratio']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['maintenance_margin_ratio']._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['maintenance_margin_ratio']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['min_price_tick_size']._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['min_price_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTPERPETUALMARKETLAUNCH._options = None
  _MSGINSTANTPERPETUALMARKETLAUNCH._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['maker_fee_rate']._options = None
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['maker_fee_rate']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['taker_fee_rate']._options = None
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['taker_fee_rate']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['min_price_tick_size']._options = None
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['min_price_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._options = None
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH._options = None
  _MSGINSTANTBINARYOPTIONSMARKETLAUNCH._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['maker_fee_rate']._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['maker_fee_rate']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['taker_fee_rate']._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['taker_fee_rate']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['initial_margin_ratio']._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['initial_margin_ratio']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['maintenance_margin_ratio']._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['maintenance_margin_ratio']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['min_price_tick_size']._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['min_price_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH.fields_by_name['min_quantity_tick_size']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH._options = None
  _MSGINSTANTEXPIRYFUTURESMARKETLAUNCH._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGCREATESPOTMARKETORDER.fields_by_name['order']._options = None
  _MSGCREATESPOTMARKETORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _MSGCREATESPOTMARKETORDER._options = None
  _MSGCREATESPOTMARKETORDER._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGCREATESPOTMARKETORDERRESPONSE.fields_by_name['results']._options = None
  _MSGCREATESPOTMARKETORDERRESPONSE.fields_by_name['results']._serialized_options = b'\310\336\037\001'
  _MSGCREATESPOTMARKETORDERRESPONSE._options = None
  _MSGCREATESPOTMARKETORDERRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _SPOTMARKETORDERRESULTS.fields_by_name['quantity']._options = None
  _SPOTMARKETORDERRESULTS.fields_by_name['quantity']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _SPOTMARKETORDERRESULTS.fields_by_name['price']._options = None
  _SPOTMARKETORDERRESULTS.fields_by_name['price']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _SPOTMARKETORDERRESULTS.fields_by_name['fee']._options = None
  _SPOTMARKETORDERRESULTS.fields_by_name['fee']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _SPOTMARKETORDERRESULTS._options = None
  _SPOTMARKETORDERRESULTS._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGCREATEDERIVATIVELIMITORDER.fields_by_name['order']._options = None
  _MSGCREATEDERIVATIVELIMITORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _MSGCREATEDERIVATIVELIMITORDER._options = None
  _MSGCREATEDERIVATIVELIMITORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGCREATEDERIVATIVELIMITORDERRESPONSE._options = None
  _MSGCREATEDERIVATIVELIMITORDERRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGCREATEBINARYOPTIONSLIMITORDER.fields_by_name['order']._options = None
  _MSGCREATEBINARYOPTIONSLIMITORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _MSGCREATEBINARYOPTIONSLIMITORDER._options = None
  _MSGCREATEBINARYOPTIONSLIMITORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGCREATEBINARYOPTIONSLIMITORDERRESPONSE._options = None
  _MSGCREATEBINARYOPTIONSLIMITORDERRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGBATCHCREATEDERIVATIVELIMITORDERS.fields_by_name['orders']._options = None
  _MSGBATCHCREATEDERIVATIVELIMITORDERS.fields_by_name['orders']._serialized_options = b'\310\336\037\000'
  _MSGBATCHCREATEDERIVATIVELIMITORDERS._options = None
  _MSGBATCHCREATEDERIVATIVELIMITORDERS._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHCREATEDERIVATIVELIMITORDERSRESPONSE._options = None
  _MSGBATCHCREATEDERIVATIVELIMITORDERSRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGCANCELSPOTORDER._options = None
  _MSGCANCELSPOTORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHCANCELSPOTORDERS.fields_by_name['data']._options = None
  _MSGBATCHCANCELSPOTORDERS.fields_by_name['data']._serialized_options = b'\310\336\037\000'
  _MSGBATCHCANCELSPOTORDERS._options = None
  _MSGBATCHCANCELSPOTORDERS._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHCANCELSPOTORDERSRESPONSE._options = None
  _MSGBATCHCANCELSPOTORDERSRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGBATCHCANCELBINARYOPTIONSORDERS.fields_by_name['data']._options = None
  _MSGBATCHCANCELBINARYOPTIONSORDERS.fields_by_name['data']._serialized_options = b'\310\336\037\000'
  _MSGBATCHCANCELBINARYOPTIONSORDERS._options = None
  _MSGBATCHCANCELBINARYOPTIONSORDERS._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHCANCELBINARYOPTIONSORDERSRESPONSE._options = None
  _MSGBATCHCANCELBINARYOPTIONSORDERSRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGBATCHUPDATEORDERS.fields_by_name['spot_orders_to_cancel']._options = None
  _MSGBATCHUPDATEORDERS.fields_by_name['spot_orders_to_cancel']._serialized_options = b'\310\336\037\001'
  _MSGBATCHUPDATEORDERS.fields_by_name['derivative_orders_to_cancel']._options = None
  _MSGBATCHUPDATEORDERS.fields_by_name['derivative_orders_to_cancel']._serialized_options = b'\310\336\037\001'
  _MSGBATCHUPDATEORDERS.fields_by_name['spot_orders_to_create']._options = None
  _MSGBATCHUPDATEORDERS.fields_by_name['spot_orders_to_create']._serialized_options = b'\310\336\037\001'
  _MSGBATCHUPDATEORDERS.fields_by_name['derivative_orders_to_create']._options = None
  _MSGBATCHUPDATEORDERS.fields_by_name['derivative_orders_to_create']._serialized_options = b'\310\336\037\001'
  _MSGBATCHUPDATEORDERS.fields_by_name['binary_options_orders_to_cancel']._options = None
  _MSGBATCHUPDATEORDERS.fields_by_name['binary_options_orders_to_cancel']._serialized_options = b'\310\336\037\001'
  _MSGBATCHUPDATEORDERS.fields_by_name['binary_options_orders_to_create']._options = None
  _MSGBATCHUPDATEORDERS.fields_by_name['binary_options_orders_to_create']._serialized_options = b'\310\336\037\001'
  _MSGBATCHUPDATEORDERS._options = None
  _MSGBATCHUPDATEORDERS._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHUPDATEORDERSRESPONSE._options = None
  _MSGBATCHUPDATEORDERSRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGCREATEDERIVATIVEMARKETORDER.fields_by_name['order']._options = None
  _MSGCREATEDERIVATIVEMARKETORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _MSGCREATEDERIVATIVEMARKETORDER._options = None
  _MSGCREATEDERIVATIVEMARKETORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGCREATEDERIVATIVEMARKETORDERRESPONSE.fields_by_name['results']._options = None
  _MSGCREATEDERIVATIVEMARKETORDERRESPONSE.fields_by_name['results']._serialized_options = b'\310\336\037\001'
  _MSGCREATEDERIVATIVEMARKETORDERRESPONSE._options = None
  _MSGCREATEDERIVATIVEMARKETORDERRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['quantity']._options = None
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['quantity']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['price']._options = None
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['price']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['fee']._options = None
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['fee']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['position_delta']._options = None
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['position_delta']._serialized_options = b'\310\336\037\000'
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['payout']._options = None
  _DERIVATIVEMARKETORDERRESULTS.fields_by_name['payout']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _DERIVATIVEMARKETORDERRESULTS._options = None
  _DERIVATIVEMARKETORDERRESULTS._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGCREATEBINARYOPTIONSMARKETORDER.fields_by_name['order']._options = None
  _MSGCREATEBINARYOPTIONSMARKETORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _MSGCREATEBINARYOPTIONSMARKETORDER._options = None
  _MSGCREATEBINARYOPTIONSMARKETORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGCREATEBINARYOPTIONSMARKETORDERRESPONSE.fields_by_name['results']._options = None
  _MSGCREATEBINARYOPTIONSMARKETORDERRESPONSE.fields_by_name['results']._serialized_options = b'\310\336\037\001'
  _MSGCREATEBINARYOPTIONSMARKETORDERRESPONSE._options = None
  _MSGCREATEBINARYOPTIONSMARKETORDERRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGCANCELDERIVATIVEORDER._options = None
  _MSGCANCELDERIVATIVEORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGCANCELBINARYOPTIONSORDER._options = None
  _MSGCANCELBINARYOPTIONSORDER._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHCANCELDERIVATIVEORDERS.fields_by_name['data']._options = None
  _MSGBATCHCANCELDERIVATIVEORDERS.fields_by_name['data']._serialized_options = b'\310\336\037\000'
  _MSGBATCHCANCELDERIVATIVEORDERS._options = None
  _MSGBATCHCANCELDERIVATIVEORDERS._serialized_options = b'\210\240\037\000\202\347\260*\006sender'
  _MSGBATCHCANCELDERIVATIVEORDERSRESPONSE._options = None
  _MSGBATCHCANCELDERIVATIVEORDERSRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000'
  _MSGSUBACCOUNTTRANSFER.fields_by_name['amount']._options = None
  _MSGSUBACCOUNTTRANSFER.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGSUBACCOUNTTRANSFER._options = None
  _MSGSUBACCOUNTTRANSFER._serialized_options = b'\202\347\260*\006sender'
  _MSGEXTERNALTRANSFER.fields_by_name['amount']._options = None
  _MSGEXTERNALTRANSFER.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _MSGEXTERNALTRANSFER._options = None
  _MSGEXTERNALTRANSFER._serialized_options = b'\202\347\260*\006sender'
  _MSGLIQUIDATEPOSITION.fields_by_name['order']._options = None
  _MSGLIQUIDATEPOSITION.fields_by_name['order']._serialized_options = b'\310\336\037\001'
  _MSGLIQUIDATEPOSITION._options = None
  _MSGLIQUIDATEPOSITION._serialized_options = b'\202\347\260*\006sender'
  _MSGINCREASEPOSITIONMARGIN.fields_by_name['amount']._options = None
  _MSGINCREASEPOSITIONMARGIN.fields_by_name['amount']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGINCREASEPOSITIONMARGIN._options = None
  _MSGINCREASEPOSITIONMARGIN._serialized_options = b'\202\347\260*\006sender'
  _MSGPRIVILEGEDEXECUTECONTRACT._options = None
  _MSGPRIVILEGEDEXECUTECONTRACT._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGPRIVILEGEDEXECUTECONTRACTRESPONSE.fields_by_name['funds_diff']._options = None
  _MSGPRIVILEGEDEXECUTECONTRACTRESPONSE.fields_by_name['funds_diff']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _MSGPRIVILEGEDEXECUTECONTRACTRESPONSE._options = None
  _MSGPRIVILEGEDEXECUTECONTRACTRESPONSE._serialized_options = b'\210\240\037\000\350\240\037\000\202\347\260*\006sender'
  _MSGRECLAIMLOCKEDFUNDS._options = None
  _MSGRECLAIMLOCKEDFUNDS._serialized_options = b'\202\347\260*\006sender'
  _MSGSIGNDATA.fields_by_name['Signer']._options = None
  _MSGSIGNDATA.fields_by_name['Signer']._serialized_options = b'\352\336\037\006signer\372\336\037-github.com/cosmos/cosmos-sdk/types.AccAddress'
  _MSGSIGNDATA.fields_by_name['Data']._options = None
  _MSGSIGNDATA.fields_by_name['Data']._serialized_options = b'\352\336\037\004data'
  _MSGSIGNDOC.fields_by_name['sign_type']._options = None
  _MSGSIGNDOC.fields_by_name['sign_type']._serialized_options = b'\352\336\037\004type'
  _MSGSIGNDOC.fields_by_name['value']._options = None
  _MSGSIGNDOC.fields_by_name['value']._serialized_options = b'\310\336\037\000'
  _MSGADMINUPDATEBINARYOPTIONSMARKET.fields_by_name['settlement_price']._options = None
  _MSGADMINUPDATEBINARYOPTIONSMARKET.fields_by_name['settlement_price']._serialized_options = b'\310\336\037\001\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _MSGADMINUPDATEBINARYOPTIONSMARKET._options = None
  _MSGADMINUPDATEBINARYOPTIONSMARKET._serialized_options = b'\202\347\260*\006sender'
  _globals['_MSGUPDATEPARAMS']._serialized_start=304
  _globals['_MSGUPDATEPARAMS']._serialized_end=440
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_start=442
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_end=467
  _globals['_MSGDEPOSIT']._serialized_start=469
  _globals['_MSGDEPOSIT']._serialized_end=590
  _globals['_MSGDEPOSITRESPONSE']._serialized_start=592
  _globals['_MSGDEPOSITRESPONSE']._serialized_end=612
  _globals['_MSGWITHDRAW']._serialized_start=614
  _globals['_MSGWITHDRAW']._serialized_end=736
  _globals['_MSGWITHDRAWRESPONSE']._serialized_start=738
  _globals['_MSGWITHDRAWRESPONSE']._serialized_end=759
  _globals['_MSGCREATESPOTLIMITORDER']._serialized_start=761
  _globals['_MSGCREATESPOTLIMITORDER']._serialized_end=883
  _globals['_MSGCREATESPOTLIMITORDERRESPONSE']._serialized_start=885
  _globals['_MSGCREATESPOTLIMITORDERRESPONSE']._serialized_end=948
  _globals['_MSGBATCHCREATESPOTLIMITORDERS']._serialized_start=951
  _globals['_MSGBATCHCREATESPOTLIMITORDERS']._serialized_end=1080
  _globals['_MSGBATCHCREATESPOTLIMITORDERSRESPONSE']._serialized_start=1082
  _globals['_MSGBATCHCREATESPOTLIMITORDERSRESPONSE']._serialized_end=1153
  _globals['_MSGINSTANTSPOTMARKETLAUNCH']._serialized_start=1156
  _globals['_MSGINSTANTSPOTMARKETLAUNCH']._serialized_end=1435
  _globals['_MSGINSTANTSPOTMARKETLAUNCHRESPONSE']._serialized_start=1437
  _globals['_MSGINSTANTSPOTMARKETLAUNCHRESPONSE']._serialized_end=1473
  _globals['_MSGINSTANTPERPETUALMARKETLAUNCH']._serialized_start=1476
  _globals['_MSGINSTANTPERPETUALMARKETLAUNCH']._serialized_end=2175
  _globals['_MSGINSTANTPERPETUALMARKETLAUNCHRESPONSE']._serialized_start=2177
  _globals['_MSGINSTANTPERPETUALMARKETLAUNCHRESPONSE']._serialized_end=2218
  _globals['_MSGINSTANTBINARYOPTIONSMARKETLAUNCH']._serialized_start=2221
  _globals['_MSGINSTANTBINARYOPTIONSMARKETLAUNCH']._serialized_end=2844
  _globals['_MSGINSTANTBINARYOPTIONSMARKETLAUNCHRESPONSE']._serialized_start=2846
  _globals['_MSGINSTANTBINARYOPTIONSMARKETLAUNCHRESPONSE']._serialized_end=2891
  _globals['_MSGINSTANTEXPIRYFUTURESMARKETLAUNCH']._serialized_start=2894
  _globals['_MSGINSTANTEXPIRYFUTURESMARKETLAUNCH']._serialized_end=3613
  _globals['_MSGINSTANTEXPIRYFUTURESMARKETLAUNCHRESPONSE']._serialized_start=3615
  _globals['_MSGINSTANTEXPIRYFUTURESMARKETLAUNCHRESPONSE']._serialized_end=3660
  _globals['_MSGCREATESPOTMARKETORDER']._serialized_start=3662
  _globals['_MSGCREATESPOTMARKETORDER']._serialized_end=3785
  _globals['_MSGCREATESPOTMARKETORDERRESPONSE']._serialized_start=3788
  _globals['_MSGCREATESPOTMARKETORDERRESPONSE']._serialized_end=3927
  _globals['_SPOTMARKETORDERRESULTS']._serialized_start=3930
  _globals['_SPOTMARKETORDERRESULTS']._serialized_end=4154
  _globals['_MSGCREATEDERIVATIVELIMITORDER']._serialized_start=4157
  _globals['_MSGCREATEDERIVATIVELIMITORDER']._serialized_end=4287
  _globals['_MSGCREATEDERIVATIVELIMITORDERRESPONSE']._serialized_start=4289
  _globals['_MSGCREATEDERIVATIVELIMITORDERRESPONSE']._serialized_end=4358
  _globals['_MSGCREATEBINARYOPTIONSLIMITORDER']._serialized_start=4361
  _globals['_MSGCREATEBINARYOPTIONSLIMITORDER']._serialized_end=4494
  _globals['_MSGCREATEBINARYOPTIONSLIMITORDERRESPONSE']._serialized_start=4496
  _globals['_MSGCREATEBINARYOPTIONSLIMITORDERRESPONSE']._serialized_end=4568
  _globals['_MSGBATCHCREATEDERIVATIVELIMITORDERS']._serialized_start=4571
  _globals['_MSGBATCHCREATEDERIVATIVELIMITORDERS']._serialized_end=4708
  _globals['_MSGBATCHCREATEDERIVATIVELIMITORDERSRESPONSE']._serialized_start=4710
  _globals['_MSGBATCHCREATEDERIVATIVELIMITORDERSRESPONSE']._serialized_end=4787
  _globals['_MSGCANCELSPOTORDER']._serialized_start=4790
  _globals['_MSGCANCELSPOTORDER']._serialized_end=4918
  _globals['_MSGCANCELSPOTORDERRESPONSE']._serialized_start=4920
  _globals['_MSGCANCELSPOTORDERRESPONSE']._serialized_end=4948
  _globals['_MSGBATCHCANCELSPOTORDERS']._serialized_start=4950
  _globals['_MSGBATCHCANCELSPOTORDERS']._serialized_end=5068
  _globals['_MSGBATCHCANCELSPOTORDERSRESPONSE']._serialized_start=5070
  _globals['_MSGBATCHCANCELSPOTORDERSRESPONSE']._serialized_end=5131
  _globals['_MSGBATCHCANCELBINARYOPTIONSORDERS']._serialized_start=5133
  _globals['_MSGBATCHCANCELBINARYOPTIONSORDERS']._serialized_end=5260
  _globals['_MSGBATCHCANCELBINARYOPTIONSORDERSRESPONSE']._serialized_start=5262
  _globals['_MSGBATCHCANCELBINARYOPTIONSORDERSRESPONSE']._serialized_end=5332
  _globals['_MSGBATCHUPDATEORDERS']._serialized_start=5335
  _globals['_MSGBATCHUPDATEORDERS']._serialized_end=6046
  _globals['_MSGBATCHUPDATEORDERSRESPONSE']._serialized_start=6049
  _globals['_MSGBATCHUPDATEORDERSRESPONSE']._serialized_end=6289
  _globals['_MSGCREATEDERIVATIVEMARKETORDER']._serialized_start=6292
  _globals['_MSGCREATEDERIVATIVEMARKETORDER']._serialized_end=6423
  _globals['_MSGCREATEDERIVATIVEMARKETORDERRESPONSE']._serialized_start=6426
  _globals['_MSGCREATEDERIVATIVEMARKETORDERRESPONSE']._serialized_end=6577
  _globals['_DERIVATIVEMARKETORDERRESULTS']._serialized_start=6580
  _globals['_DERIVATIVEMARKETORDERRESULTS']._serialized_end=6947
  _globals['_MSGCREATEBINARYOPTIONSMARKETORDER']._serialized_start=6950
  _globals['_MSGCREATEBINARYOPTIONSMARKETORDER']._serialized_end=7084
  _globals['_MSGCREATEBINARYOPTIONSMARKETORDERRESPONSE']._serialized_start=7087
  _globals['_MSGCREATEBINARYOPTIONSMARKETORDERRESPONSE']._serialized_end=7241
  _globals['_MSGCANCELDERIVATIVEORDER']._serialized_start=7244
  _globals['_MSGCANCELDERIVATIVEORDER']._serialized_end=7398
  _globals['_MSGCANCELDERIVATIVEORDERRESPONSE']._serialized_start=7400
  _globals['_MSGCANCELDERIVATIVEORDERRESPONSE']._serialized_end=7434
  _globals['_MSGCANCELBINARYOPTIONSORDER']._serialized_start=7437
  _globals['_MSGCANCELBINARYOPTIONSORDER']._serialized_end=7594
  _globals['_MSGCANCELBINARYOPTIONSORDERRESPONSE']._serialized_start=7596
  _globals['_MSGCANCELBINARYOPTIONSORDERRESPONSE']._serialized_end=7633
  _globals['_ORDERDATA']._serialized_start=7635
  _globals['_ORDERDATA']._serialized_end=7741
  _globals['_MSGBATCHCANCELDERIVATIVEORDERS']._serialized_start=7743
  _globals['_MSGBATCHCANCELDERIVATIVEORDERS']._serialized_end=7867
  _globals['_MSGBATCHCANCELDERIVATIVEORDERSRESPONSE']._serialized_start=7869
  _globals['_MSGBATCHCANCELDERIVATIVEORDERSRESPONSE']._serialized_end=7936
  _globals['_MSGSUBACCOUNTTRANSFER']._serialized_start=7939
  _globals['_MSGSUBACCOUNTTRANSFER']._serialized_end=8105
  _globals['_MSGSUBACCOUNTTRANSFERRESPONSE']._serialized_start=8107
  _globals['_MSGSUBACCOUNTTRANSFERRESPONSE']._serialized_end=8138
  _globals['_MSGEXTERNALTRANSFER']._serialized_start=8141
  _globals['_MSGEXTERNALTRANSFER']._serialized_end=8305
  _globals['_MSGEXTERNALTRANSFERRESPONSE']._serialized_start=8307
  _globals['_MSGEXTERNALTRANSFERRESPONSE']._serialized_end=8336
  _globals['_MSGLIQUIDATEPOSITION']._serialized_start=8339
  _globals['_MSGLIQUIDATEPOSITION']._serialized_end=8498
  _globals['_MSGLIQUIDATEPOSITIONRESPONSE']._serialized_start=8500
  _globals['_MSGLIQUIDATEPOSITIONRESPONSE']._serialized_end=8530
  _globals['_MSGINCREASEPOSITIONMARGIN']._serialized_start=8533
  _globals['_MSGINCREASEPOSITIONMARGIN']._serialized_end=8737
  _globals['_MSGINCREASEPOSITIONMARGINRESPONSE']._serialized_start=8739
  _globals['_MSGINCREASEPOSITIONMARGINRESPONSE']._serialized_end=8774
  _globals['_MSGPRIVILEGEDEXECUTECONTRACT']._serialized_start=8776
  _globals['_MSGPRIVILEGEDEXECUTECONTRACT']._serialized_end=8898
  _globals['_MSGPRIVILEGEDEXECUTECONTRACTRESPONSE']._serialized_start=8901
  _globals['_MSGPRIVILEGEDEXECUTECONTRACTRESPONSE']._serialized_end=9057
  _globals['_MSGREWARDSOPTOUT']._serialized_start=9059
  _globals['_MSGREWARDSOPTOUT']._serialized_end=9093
  _globals['_MSGREWARDSOPTOUTRESPONSE']._serialized_start=9095
  _globals['_MSGREWARDSOPTOUTRESPONSE']._serialized_end=9121
  _globals['_MSGRECLAIMLOCKEDFUNDS']._serialized_start=9123
  _globals['_MSGRECLAIMLOCKEDFUNDS']._serialized_end=9223
  _globals['_MSGRECLAIMLOCKEDFUNDSRESPONSE']._serialized_start=9225
  _globals['_MSGRECLAIMLOCKEDFUNDSRESPONSE']._serialized_end=9256
  _globals['_MSGSIGNDATA']._serialized_start=9258
  _globals['_MSGSIGNDATA']._serialized_end=9372
  _globals['_MSGSIGNDOC']._serialized_start=9374
  _globals['_MSGSIGNDOC']._serialized_end=9477
  _globals['_MSGADMINUPDATEBINARYOPTIONSMARKET']._serialized_start=9480
  _globals['_MSGADMINUPDATEBINARYOPTIONSMARKET']._serialized_end=9755
  _globals['_MSGADMINUPDATEBINARYOPTIONSMARKETRESPONSE']._serialized_start=9757
  _globals['_MSGADMINUPDATEBINARYOPTIONSMARKETRESPONSE']._serialized_end=9800
  _globals['_MSG']._serialized_start=9803
  _globals['_MSG']._serialized_end=14171
# @@protoc_insertion_point(module_scope)



import json
from configparser import ConfigParser
from decimal import Decimal
from time import time
from typing import Dict, List, Optional

from google.protobuf import any_pb2, json_format, timestamp_pb2

from pyinjective import constant
from pyinjective.proto.cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from pyinjective.proto.injective.exchange.v1beta1 import (
    exchange_pb2 as injective_dot_exchange_dot_v1beta1_dot_exchange__pb2,
)
from pyinjective.proto.injective.stream.v1beta1 import query_pb2 as chain_stream_query

from .constant import ADDITIONAL_CHAIN_FORMAT_DECIMALS, INJ_DENOM
from .core.market import BinaryOptionMarket, DerivativeMarket, SpotMarket
from .core.token import Token
from .proto.cosmos.authz.v1beta1 import authz_pb2 as cosmos_authz_pb
from .proto.cosmos.authz.v1beta1 import tx_pb2 as cosmos_authz_tx_pb
from .proto.cosmos.bank.v1beta1 import tx_pb2 as cosmos_bank_tx_pb
from .proto.cosmos.distribution.v1beta1 import tx_pb2 as cosmos_distribution_tx_pb
from .proto.cosmos.gov.v1beta1 import tx_pb2 as cosmos_gov_tx_pb
from .proto.cosmos.staking.v1beta1 import tx_pb2 as cosmos_staking_tx_pb
from .proto.cosmwasm.wasm.v1 import tx_pb2 as wasm_tx_pb
from .proto.injective.auction.v1beta1 import tx_pb2 as injective_auction_tx_pb
from .proto.injective.exchange.v1beta1 import authz_pb2 as injective_authz_pb
from .proto.injective.exchange.v1beta1 import tx_pb2 as injective_exchange_tx_pb
from .proto.injective.insurance.v1beta1 import tx_pb2 as injective_insurance_tx_pb
from .proto.injective.oracle.v1beta1 import tx_pb2 as injective_oracle_tx_pb
from .proto.injective.peggy.v1 import msgs_pb2 as injective_peggy_tx_pb


class Composer:
    def __init__(
        self,
        network: str,
        spot_markets: Optional[Dict[str, SpotMarket]] = None,
        derivative_markets: Optional[Dict[str, DerivativeMarket]] = None,
        binary_option_markets: Optional[Dict[str, BinaryOptionMarket]] = None,
        tokens: Optional[Dict[str, Token]] = None,
    ):
        """Composer is used to create the requests to send to the nodes using the Client

        :param network: the name of the network to use (mainnet, testnet, devnet)
        :type network: str

        :param spot_markets: a dictionary containing all spot markets
        :type spot_markets: Dictionary

        :param derivative_markets: a dictionary containing all derivative markets
        :type derivative_markets: Dictionary

        :param binary_option_markets: a dictionary containing all derivative markets
        :type binary_option_markets: Dictionary

        :param tokens: a dictionary with all the possible tokens
        :type tokens: Dictionary


        """
        self.network = network
        self.spot_markets = dict()
        self.derivative_markets = dict()
        self.binary_option_markets = dict()
        self.tokens = dict()
        if spot_markets is None or derivative_markets is None or binary_option_markets is None or tokens is None:
            self._initialize_markets_and_tokens_from_files()
        else:
            self.spot_markets = spot_markets
            self.derivative_markets = derivative_markets
            self.binary_option_markets = binary_option_markets
            self.tokens = tokens

    def Coin(self, amount: int, denom: str):
        return cosmos_dot_base_dot_v1beta1_dot_coin__pb2.Coin(amount=str(amount), denom=denom)

    def get_order_mask(self, **kwargs):
        order_mask = 0

        if kwargs.get("is_conditional"):
            order_mask += injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderMask.CONDITIONAL
        else:
            order_mask += injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderMask.REGULAR

        if kwargs.get("order_direction") == "buy":
            order_mask += injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderMask.DIRECTION_BUY_OR_HIGHER

        elif kwargs.get("order_direction") == "sell":
            order_mask += injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderMask.DIRECTION_SELL_OR_LOWER

        if kwargs.get("order_type") == "market":
            order_mask += injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderMask.TYPE_MARKET

        elif kwargs.get("order_type") == "limit":
            order_mask += injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderMask.TYPE_LIMIT

        if order_mask == 0:
            order_mask = 1

        return order_mask

    def OrderData(
        self, market_id: str, subaccount_id: str, order_hash: Optional[str] = None, cid: Optional[str] = None, **kwargs
    ):
        order_mask = self.get_order_mask(**kwargs)

        return injective_exchange_tx_pb.OrderData(
            market_id=market_id,
            subaccount_id=subaccount_id,
            order_hash=order_hash,
            order_mask=order_mask,
            cid=cid,
        )

    def SpotOrder(
        self,
        market_id: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        cid: Optional[str] = None,
        **kwargs,
    ):
        market = self.spot_markets[market_id]

        # prepare values
        quantity = market.quantity_to_chain_format(human_readable_value=Decimal(str(quantity)))
        price = market.price_to_chain_format(human_readable_value=Decimal(str(price)))
        trigger_price = market.price_to_chain_format(human_readable_value=Decimal(0))

        if kwargs.get("is_buy") and not kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.BUY

        elif not kwargs.get("is_buy") and not kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.SELL

        elif kwargs.get("is_buy") and kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.BUY_PO

        elif not kwargs.get("is_buy") and kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.SELL_PO

        return injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.SpotOrder(
            market_id=market_id,
            order_info=injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderInfo(
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=str(int(price)),
                quantity=str(int(quantity)),
                cid=cid,
            ),
            order_type=order_type,
            trigger_price=str(int(trigger_price)),
        )

    def DerivativeOrder(
        self,
        market_id: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        trigger_price: float = 0,
        cid: Optional[str] = None,
        **kwargs,
    ):
        market = self.derivative_markets[market_id]

        if kwargs.get("is_reduce_only", False):
            margin = 0
        else:
            margin = market.calculate_margin_in_chain_format(
                human_readable_quantity=Decimal(str(quantity)),
                human_readable_price=Decimal(str(price)),
                leverage=Decimal(str(kwargs["leverage"])),
            )

        # prepare values
        quantity = market.quantity_to_chain_format(human_readable_value=Decimal(str(quantity)))
        price = market.price_to_chain_format(human_readable_value=Decimal(str(price)))
        trigger_price = market.price_to_chain_format(human_readable_value=Decimal(str(trigger_price)))

        if kwargs.get("is_buy") and not kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.BUY

        elif not kwargs.get("is_buy") and not kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.SELL

        elif kwargs.get("is_buy") and kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.BUY_PO

        elif not kwargs.get("is_buy") and kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.SELL_PO

        elif kwargs.get("stop_buy"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.STOP_BUY

        elif kwargs.get("stop_sell"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.STOP_SEll

        elif kwargs.get("take_buy"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.TAKE_BUY

        elif kwargs.get("take_sell"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.TAKE_SELL

        return injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.DerivativeOrder(
            market_id=market_id,
            order_info=injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderInfo(
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=str(int(price)),
                quantity=str(int(quantity)),
                cid=cid,
            ),
            margin=str(int(margin)),
            order_type=order_type,
            trigger_price=str(int(trigger_price)),
        )

    def BinaryOptionsOrder(
        self,
        market_id: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        cid: Optional[str] = None,
        **kwargs,
    ):
        market = self.binary_option_markets[market_id]
        denom = kwargs.get("denom", None)

        if kwargs.get("is_reduce_only", False):
            margin = 0
        else:
            margin = market.calculate_margin_in_chain_format(
                human_readable_quantity=Decimal(str(quantity)),
                human_readable_price=Decimal(str(price)),
                is_buy=kwargs["is_buy"],
                special_denom=denom,
            )

        # prepare values
        price = market.price_to_chain_format(human_readable_value=Decimal(str(price)), special_denom=denom)
        trigger_price = market.price_to_chain_format(human_readable_value=Decimal(str(0)), special_denom=denom)
        quantity = market.quantity_to_chain_format(human_readable_value=Decimal(str(quantity)), special_denom=denom)

        if kwargs.get("is_buy") and not kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.BUY

        elif not kwargs.get("is_buy") and not kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.SELL

        elif kwargs.get("is_buy") and kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.BUY_PO

        elif not kwargs.get("is_buy") and kwargs.get("is_po"):
            order_type = injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderType.SELL_PO

        return injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.DerivativeOrder(
            market_id=market_id,
            order_info=injective_dot_exchange_dot_v1beta1_dot_exchange__pb2.OrderInfo(
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=str(int(price)),
                quantity=str(int(quantity)),
                cid=cid,
            ),
            margin=str(int(margin)),
            order_type=order_type,
            trigger_price=str(int(trigger_price)),
        )

    def MsgSend(self, from_address: str, to_address: str, amount: float, denom: str):
        token = self.tokens[denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))

        return cosmos_bank_tx_pb.MsgSend(
            from_address=from_address,
            to_address=to_address,
            amount=[self.Coin(amount=int(be_amount), denom=token.denom)],
        )

    def MsgExecuteContract(self, sender: str, contract: str, msg: str, **kwargs):
        return wasm_tx_pb.MsgExecuteContract(
            sender=sender,
            contract=contract,
            msg=bytes(msg, "utf-8"),
            funds=kwargs.get("funds")  # funds is a list of cosmos_dot_base_dot_v1beta1_dot_coin__pb2.Coin.
            # The coins in the list must be sorted in alphabetical order by denoms.
        )

    def MsgDeposit(self, sender: str, subaccount_id: str, amount: float, denom: str):
        token = self.tokens[denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))

        return injective_exchange_tx_pb.MsgDeposit(
            sender=sender,
            subaccount_id=subaccount_id,
            amount=self.Coin(amount=int(be_amount), denom=token.denom),
        )

    def MsgCreateSpotLimitOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        cid: Optional[str] = None,
        **kwargs,
    ):
        return injective_exchange_tx_pb.MsgCreateSpotLimitOrder(
            sender=sender,
            order=self.SpotOrder(
                market_id=market_id,
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=price,
                quantity=quantity,
                cid=cid,
                **kwargs,
            ),
        )

    def MsgCreateSpotMarketOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        is_buy: bool,
        cid: Optional[str] = None,
    ):
        return injective_exchange_tx_pb.MsgCreateSpotMarketOrder(
            sender=sender,
            order=self.SpotOrder(
                market_id=market_id,
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=price,
                quantity=quantity,
                is_buy=is_buy,
                cid=cid,
            ),
        )

    def MsgCancelSpotOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        order_hash: Optional[str] = None,
        cid: Optional[str] = None,
    ):
        return injective_exchange_tx_pb.MsgCancelSpotOrder(
            sender=sender,
            market_id=market_id,
            subaccount_id=subaccount_id,
            order_hash=order_hash,
            cid=cid,
        )

    def MsgBatchCreateSpotLimitOrders(self, sender: str, orders: List):
        return injective_exchange_tx_pb.MsgBatchCreateSpotLimitOrders(sender=sender, orders=orders)

    def MsgBatchCancelSpotOrders(self, sender: str, data: List):
        return injective_exchange_tx_pb.MsgBatchCancelSpotOrders(sender=sender, data=data)

    def MsgRewardsOptOut(self, sender: str):
        return injective_exchange_tx_pb.MsgRewardsOptOut(sender=sender)

    def MsgCreateDerivativeLimitOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        cid: Optional[str] = None,
        **kwargs,
    ):
        return injective_exchange_tx_pb.MsgCreateDerivativeLimitOrder(
            sender=sender,
            order=self.DerivativeOrder(
                market_id=market_id,
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=price,
                quantity=quantity,
                cid=cid,
                **kwargs,
            ),
        )

    def MsgCreateDerivativeMarketOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        is_buy: bool,
        cid: Optional[str] = None,
        **kwargs,
    ):
        return injective_exchange_tx_pb.MsgCreateDerivativeMarketOrder(
            sender=sender,
            order=self.DerivativeOrder(
                market_id=market_id,
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=price,
                quantity=quantity,
                is_buy=is_buy,
                cid=cid,
                **kwargs,
            ),
        )

    def MsgCreateBinaryOptionsLimitOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        cid: Optional[str] = None,
        **kwargs,
    ):
        return injective_exchange_tx_pb.MsgCreateBinaryOptionsLimitOrder(
            sender=sender,
            order=self.BinaryOptionsOrder(
                market_id=market_id,
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=price,
                quantity=quantity,
                cid=cid,
                **kwargs,
            ),
        )

    def MsgCreateBinaryOptionsMarketOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        fee_recipient: str,
        price: float,
        quantity: float,
        cid: Optional[str] = None,
        **kwargs,
    ):
        return injective_exchange_tx_pb.MsgCreateBinaryOptionsMarketOrder(
            sender=sender,
            order=self.BinaryOptionsOrder(
                market_id=market_id,
                subaccount_id=subaccount_id,
                fee_recipient=fee_recipient,
                price=price,
                quantity=quantity,
                cid=cid,
                **kwargs,
            ),
        )

    def MsgCancelBinaryOptionsOrder(
        self,
        sender: str,
        market_id: str,
        subaccount_id: str,
        order_hash: Optional[str] = None,
        cid: Optional[str] = None,
    ):
        return injective_exchange_tx_pb.MsgCancelBinaryOptionsOrder(
            sender=sender,
            market_id=market_id,
            subaccount_id=subaccount_id,
            order_hash=order_hash,
            cid=cid,
        )

    def MsgAdminUpdateBinaryOptionsMarket(
        self,
        sender: str,
        market_id: str,
        status: str,
        **kwargs,
    ):
        price_to_bytes = None

        if kwargs.get("settlement_price") is not None:
            scale_price = Decimal((kwargs.get("settlement_price") * pow(10, 18)))
            price_to_bytes = bytes(str(scale_price), "utf-8")

        else:
            price_to_bytes = ""

        return injective_exchange_tx_pb.MsgAdminUpdateBinaryOptionsMarket(
            sender=sender,
            market_id=market_id,
            settlement_price=price_to_bytes,
            expiration_timestamp=kwargs.get("expiration_timestamp"),
            settlement_timestamp=kwargs.get("settlement_timestamp"),
            status=status,
        )

    def MsgRelayProviderPrices(self, sender: str, provider: str, symbols: list, prices: list):
        oracle_prices = []

        for price in prices:
            scale_price = Decimal((price) * pow(10, 18))
            price_to_bytes = bytes(str(scale_price), "utf-8")
            oracle_prices.append(price_to_bytes)

        return injective_oracle_tx_pb.MsgRelayProviderPrices(
            sender=sender, provider=provider, symbols=symbols, prices=oracle_prices
        )

    def MsgInstantBinaryOptionsMarketLaunch(
        self,
        sender: str,
        ticker: str,
        oracle_symbol: str,
        oracle_provider: str,
        oracle_type: str,
        oracle_scale_factor: int,
        maker_fee_rate: float,
        taker_fee_rate: float,
        expiration_timestamp: int,
        settlement_timestamp: int,
        quote_denom: str,
        quote_decimals: int,
        min_price_tick_size: float,
        min_quantity_tick_size: float,
        **kwargs,
    ):
        scaled_maker_fee_rate = Decimal((maker_fee_rate * pow(10, 18)))
        maker_fee_to_bytes = bytes(str(scaled_maker_fee_rate), "utf-8")

        scaled_taker_fee_rate = Decimal((taker_fee_rate * pow(10, 18)))
        taker_fee_to_bytes = bytes(str(scaled_taker_fee_rate), "utf-8")

        scaled_min_price_tick_size = Decimal((min_price_tick_size * pow(10, quote_decimals + 18)))
        min_price_to_bytes = bytes(str(scaled_min_price_tick_size), "utf-8")

        scaled_min_quantity_tick_size = Decimal((min_quantity_tick_size * pow(10, 18)))
        min_quantity_to_bytes = bytes(str(scaled_min_quantity_tick_size), "utf-8")

        return injective_exchange_tx_pb.MsgInstantBinaryOptionsMarketLaunch(
            sender=sender,
            ticker=ticker,
            oracle_symbol=oracle_symbol,
            oracle_provider=oracle_provider,
            oracle_type=oracle_type,
            oracle_scale_factor=oracle_scale_factor,
            maker_fee_rate=maker_fee_to_bytes,
            taker_fee_rate=taker_fee_to_bytes,
            expiration_timestamp=expiration_timestamp,
            settlement_timestamp=settlement_timestamp,
            quote_denom=quote_denom,
            min_price_tick_size=min_price_to_bytes,
            min_quantity_tick_size=min_quantity_to_bytes,
            admin=kwargs.get("admin"),
        )

    def MsgCancelDerivativeOrder(
        self,
        market_id: str,
        sender: str,
        subaccount_id: str,
        order_hash: Optional[str] = None,
        cid: Optional[str] = None,
        **kwargs,
    ):
        order_mask = self.get_order_mask(**kwargs)

        return injective_exchange_tx_pb.MsgCancelDerivativeOrder(
            sender=sender,
            market_id=market_id,
            subaccount_id=subaccount_id,
            order_hash=order_hash,
            order_mask=order_mask,
            cid=cid,
        )

    def MsgBatchCreateDerivativeLimitOrders(self, sender: str, orders: List):
        return injective_exchange_tx_pb.MsgBatchCreateDerivativeLimitOrders(sender=sender, orders=orders)

    def MsgBatchCancelDerivativeOrders(self, sender: str, data: List):
        return injective_exchange_tx_pb.MsgBatchCancelDerivativeOrders(sender=sender, data=data)

    def MsgBatchUpdateOrders(self, sender: str, **kwargs):
        return injective_exchange_tx_pb.MsgBatchUpdateOrders(
            sender=sender,
            subaccount_id=kwargs.get("subaccount_id"),
            spot_market_ids_to_cancel_all=kwargs.get("spot_market_ids_to_cancel_all"),
            derivative_market_ids_to_cancel_all=kwargs.get("derivative_market_ids_to_cancel_all"),
            spot_orders_to_cancel=kwargs.get("spot_orders_to_cancel"),
            derivative_orders_to_cancel=kwargs.get("derivative_orders_to_cancel"),
            spot_orders_to_create=kwargs.get("spot_orders_to_create"),
            derivative_orders_to_create=kwargs.get("derivative_orders_to_create"),
            binary_options_orders_to_cancel=kwargs.get("binary_options_orders_to_cancel"),
            binary_options_market_ids_to_cancel_all=kwargs.get("binary_options_market_ids_to_cancel_all"),
            binary_options_orders_to_create=kwargs.get("binary_options_orders_to_create"),
        )

    def MsgLiquidatePosition(self, sender: str, subaccount_id: str, market_id: str):
        return injective_exchange_tx_pb.MsgLiquidatePosition(
            sender=sender, subaccount_id=subaccount_id, market_id=market_id
        )

    def MsgIncreasePositionMargin(
        self,
        sender: str,
        source_subaccount_id: str,
        destination_subaccount_id: str,
        market_id: str,
        amount: float,
    ):
        market = self.derivative_markets[market_id]

        additional_margin = market.margin_to_chain_format(human_readable_value=Decimal(str(amount)))
        return injective_exchange_tx_pb.MsgIncreasePositionMargin(
            sender=sender,
            source_subaccount_id=source_subaccount_id,
            destination_subaccount_id=destination_subaccount_id,
            market_id=market_id,
            amount=str(int(additional_margin)),
        )

    def MsgSubaccountTransfer(
        self,
        sender: str,
        source_subaccount_id: str,
        destination_subaccount_id: str,
        amount: int,
        denom: str,
    ):
        token = self.tokens[denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))

        return injective_exchange_tx_pb.MsgSubaccountTransfer(
            sender=sender,
            source_subaccount_id=source_subaccount_id,
            destination_subaccount_id=destination_subaccount_id,
            amount=self.Coin(amount=int(be_amount), denom=token.denom),
        )

    def MsgWithdraw(self, sender: str, subaccount_id: str, amount: float, denom: str):
        token = self.tokens[denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))

        return injective_exchange_tx_pb.MsgWithdraw(
            sender=sender,
            subaccount_id=subaccount_id,
            amount=self.Coin(amount=int(be_amount), denom=token.denom),
        )

    def MsgExternalTransfer(
        self,
        sender: str,
        source_subaccount_id: str,
        destination_subaccount_id: str,
        amount: int,
        denom: str,
    ):
        token = self.tokens[denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))

        return injective_exchange_tx_pb.MsgExternalTransfer(
            sender=sender,
            source_subaccount_id=source_subaccount_id,
            destination_subaccount_id=destination_subaccount_id,
            amount=self.Coin(amount=int(be_amount), denom=token.denom),
        )

    def MsgBid(self, sender: str, bid_amount: float, round: float):
        be_amount = Decimal(str(bid_amount)) * Decimal(f"1e{ADDITIONAL_CHAIN_FORMAT_DECIMALS}")

        return injective_auction_tx_pb.MsgBid(
            sender=sender,
            round=round,
            bid_amount=self.Coin(amount=int(be_amount), denom=INJ_DENOM),
        )

    def MsgGrantGeneric(self, granter: str, grantee: str, msg_type: str, expire_in: int):
        auth = cosmos_authz_pb.GenericAuthorization(msg=msg_type)
        any_auth = any_pb2.Any()
        any_auth.Pack(auth, type_url_prefix="")

        grant = cosmos_authz_pb.Grant(
            authorization=any_auth,
            expiration=timestamp_pb2.Timestamp(seconds=(int(time()) + expire_in)),
        )

        return cosmos_authz_tx_pb.MsgGrant(granter=granter, grantee=grantee, grant=grant)

    def MsgGrantTyped(
        self,
        granter: str,
        grantee: str,
        msg_type: str,
        expire_in: int,
        subaccount_id: str,
        **kwargs,
    ):
        auth = None
        if msg_type == "CreateSpotLimitOrderAuthz":
            auth = injective_authz_pb.CreateSpotLimitOrderAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "CreateSpotMarketOrderAuthz":
            auth = injective_authz_pb.CreateSpotMarketOrderAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "BatchCreateSpotLimitOrdersAuthz":
            auth = injective_authz_pb.BatchCreateSpotLimitOrdersAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "CancelSpotOrderAuthz":
            auth = injective_authz_pb.CancelSpotOrderAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "BatchCancelSpotOrdersAuthz":
            auth = injective_authz_pb.BatchCancelSpotOrdersAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "CreateDerivativeLimitOrderAuthz":
            auth = injective_authz_pb.CreateDerivativeLimitOrderAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "CreateDerivativeMarketOrderAuthz":
            auth = injective_authz_pb.CreateDerivativeMarketOrderAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "BatchCreateDerivativeLimitOrdersAuthz":
            auth = injective_authz_pb.BatchCreateDerivativeLimitOrdersAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "CancelDerivativeOrderAuthz":
            auth = injective_authz_pb.CancelDerivativeOrderAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "BatchCancelDerivativeOrdersAuthz":
            auth = injective_authz_pb.BatchCancelDerivativeOrdersAuthz(
                subaccount_id=subaccount_id, market_ids=kwargs.get("market_ids")
            )
        elif msg_type == "BatchUpdateOrdersAuthz":
            auth = injective_authz_pb.BatchUpdateOrdersAuthz(
                subaccount_id=subaccount_id,
                spot_markets=kwargs.get("spot_markets"),
                derivative_markets=kwargs.get("derivative_markets"),
            )

        any_auth = any_pb2.Any()
        any_auth.Pack(auth, type_url_prefix="")

        grant = cosmos_authz_pb.Grant(
            authorization=any_auth,
            expiration=timestamp_pb2.Timestamp(seconds=(int(time()) + expire_in)),
        )

        return cosmos_authz_tx_pb.MsgGrant(granter=granter, grantee=grantee, grant=grant)

    def MsgExec(self, grantee: str, msgs: List):
        any_msgs: List[any_pb2.Any] = []
        for msg in msgs:
            any_msg = any_pb2.Any()
            any_msg.Pack(msg, type_url_prefix="")
            any_msgs.append(any_msg)

        return cosmos_authz_tx_pb.MsgExec(grantee=grantee, msgs=any_msgs)

    def MsgRevoke(self, granter: str, grantee: str, msg_type: str):
        return cosmos_authz_tx_pb.MsgRevoke(granter=granter, grantee=grantee, msg_type_url=msg_type)

    def MsgRelayPriceFeedPrice(self, sender: list, base: list, quote: list, price: list):
        return injective_oracle_tx_pb.MsgRelayPriceFeedPrice(sender=sender, base=base, quote=quote, price=price)

    def MsgSendToEth(self, denom: str, sender: str, eth_dest: str, amount: float, bridge_fee: float):
        token = self.tokens[denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))
        be_bridge_fee = token.chain_formatted_value(human_readable_value=Decimal(str(bridge_fee)))

        return injective_peggy_tx_pb.MsgSendToEth(
            sender=sender,
            eth_dest=eth_dest,
            amount=self.Coin(amount=int(be_amount), denom=token.denom),
            bridge_fee=self.Coin(amount=int(be_bridge_fee), denom=token.denom),
        )

    def MsgDelegate(self, delegator_address: str, validator_address: str, amount: float):
        be_amount = Decimal(str(amount)) * Decimal(f"1e{ADDITIONAL_CHAIN_FORMAT_DECIMALS}")

        return cosmos_staking_tx_pb.MsgDelegate(
            delegator_address=delegator_address,
            validator_address=validator_address,
            amount=self.Coin(amount=int(be_amount), denom=INJ_DENOM),
        )

    def MsgCreateInsuranceFund(
        self,
        sender: str,
        ticker: str,
        quote_denom: str,
        oracle_base: str,
        oracle_quote: str,
        oracle_type: int,
        expiry: int,
        initial_deposit: int,
    ):
        token = self.tokens[quote_denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(initial_deposit)))

        return injective_insurance_tx_pb.MsgCreateInsuranceFund(
            sender=sender,
            ticker=ticker,
            quote_denom=token.denom,
            oracle_base=oracle_base,
            oracle_quote=oracle_quote,
            oracle_type=oracle_type,
            expiry=expiry,
            initial_deposit=self.Coin(amount=int(be_amount), denom=token.denom),
        )

    def MsgUnderwrite(
        self,
        sender: str,
        market_id: str,
        quote_denom: str,
        amount: int,
    ):
        token = self.tokens[quote_denom]
        be_amount = token.chain_formatted_value(human_readable_value=Decimal(str(amount)))

        return injective_insurance_tx_pb.MsgUnderwrite(
            sender=sender,
            market_id=market_id,
            deposit=self.Coin(amount=int(be_amount), denom=token.denom),
        )

    def MsgRequestRedemption(
        self,
        sender: str,
        market_id: str,
        share_denom: str,
        amount: int,
    ):
        return injective_insurance_tx_pb.MsgRequestRedemption(
            sender=sender,
            market_id=market_id,
            amount=self.Coin(amount=amount, denom=share_denom),
        )

    def MsgWithdrawDelegatorReward(self, delegator_address: str, validator_address: str):
        return cosmos_distribution_tx_pb.MsgWithdrawDelegatorReward(
            delegator_address=delegator_address, validator_address=validator_address
        )

    def MsgWithdrawValidatorCommission(self, validator_address: str):
        return cosmos_distribution_tx_pb.MsgWithdrawValidatorCommission(validator_address=validator_address)

    def MsgVote(
        self,
        proposal_id: str,
        voter: str,
        option: int,
    ):
        return cosmos_gov_tx_pb.MsgVote(proposal_id=proposal_id, voter=voter, option=option)

    def MsgPrivilegedExecuteContract(
        self, sender: str, contract: str, msg: str, **kwargs
    ) -> injective_exchange_tx_pb.MsgPrivilegedExecuteContract:
        return injective_exchange_tx_pb.MsgPrivilegedExecuteContract(
            sender=sender,
            contract_address=contract,
            data=msg,
            funds=kwargs.get("funds")  # funds is a string of Coin strings, comma separated,
            # e.g. 100000inj,20000000000usdt
        )

    def MsgInstantiateContract(
        self, sender: str, admin: str, code_id: int, label: str, message: bytes, **kwargs
    ) -> wasm_tx_pb.MsgInstantiateContract:
        return wasm_tx_pb.MsgInstantiateContract(
            sender=sender,
            admin=admin,
            code_id=code_id,
            label=label,
            msg=message,
            funds=kwargs.get("funds"),  # funds is a list of cosmos_dot_base_dot_v1beta1_dot_coin__pb2.Coin.
            # The coins in the list must be sorted in alphabetical order by denoms.
        )

    def chain_stream_bank_balances_filter(
        self, accounts: Optional[List[str]] = None
    ) -> chain_stream_query.BankBalancesFilter:
        accounts = accounts or ["*"]
        return chain_stream_query.BankBalancesFilter(accounts=accounts)

    def chain_stream_subaccount_deposits_filter(
        self,
        subaccount_ids: Optional[List[str]] = None,
    ) -> chain_stream_query.SubaccountDepositsFilter:
        subaccount_ids = subaccount_ids or ["*"]
        return chain_stream_query.SubaccountDepositsFilter(subaccount_ids=subaccount_ids)

    def chain_stream_trades_filter(
        self,
        subaccount_ids: Optional[List[str]] = None,
        market_ids: Optional[List[str]] = None,
    ) -> chain_stream_query.TradesFilter:
        subaccount_ids = subaccount_ids or ["*"]
        market_ids = market_ids or ["*"]
        return chain_stream_query.TradesFilter(subaccount_ids=subaccount_ids, market_ids=market_ids)

    def chain_stream_orders_filter(
        self,
        subaccount_ids: Optional[List[str]] = None,
        market_ids: Optional[List[str]] = None,
    ) -> chain_stream_query.OrdersFilter:
        subaccount_ids = subaccount_ids or ["*"]
        market_ids = market_ids or ["*"]
        return chain_stream_query.OrdersFilter(subaccount_ids=subaccount_ids, market_ids=market_ids)

    def chain_stream_orderbooks_filter(
        self,
        market_ids: Optional[List[str]] = None,
    ) -> chain_stream_query.OrderbookFilter:
        market_ids = market_ids or ["*"]
        return chain_stream_query.OrderbookFilter(market_ids=market_ids)

    def chain_stream_positions_filter(
        self,
        subaccount_ids: Optional[List[str]] = None,
        market_ids: Optional[List[str]] = None,
    ) -> chain_stream_query.PositionsFilter:
        subaccount_ids = subaccount_ids or ["*"]
        market_ids = market_ids or ["*"]
        return chain_stream_query.PositionsFilter(subaccount_ids=subaccount_ids, market_ids=market_ids)

    def chain_stream_oracle_price_filter(
        self,
        symbols: Optional[List[str]] = None,
    ) -> chain_stream_query.PositionsFilter:
        symbols = symbols or ["*"]
        return chain_stream_query.OraclePriceFilter(symbol=symbols)

    # data field format: [request-msg-header][raw-byte-msg-response]
    # you need to figure out this magic prefix number to trim request-msg-header off the data
    # this method handles only exchange responses
    @staticmethod
    def MsgResponses(response, simulation=False):
        data = response.result
        if not simulation:
            data = bytes.fromhex(data)
        # fmt: off
        header_map = {
            "/injective.exchange.v1beta1.MsgCreateSpotLimitOrderResponse":
                injective_exchange_tx_pb.MsgCreateSpotLimitOrderResponse,
            "/injective.exchange.v1beta1.MsgCreateSpotMarketOrderResponse":
                injective_exchange_tx_pb.MsgCreateSpotMarketOrderResponse,
            "/injective.exchange.v1beta1.MsgCreateDerivativeLimitOrderResponse":
                injective_exchange_tx_pb.MsgCreateDerivativeLimitOrderResponse,
            "/injective.exchange.v1beta1.MsgCreateDerivativeMarketOrderResponse":
                injective_exchange_tx_pb.MsgCreateDerivativeMarketOrderResponse,
            "/injective.exchange.v1beta1.MsgCancelSpotOrderResponse":
                injective_exchange_tx_pb.MsgCancelSpotOrderResponse,
            "/injective.exchange.v1beta1.MsgCancelDerivativeOrderResponse":
                injective_exchange_tx_pb.MsgCancelDerivativeOrderResponse,
            "/injective.exchange.v1beta1.MsgBatchCancelSpotOrdersResponse":
                injective_exchange_tx_pb.MsgBatchCancelSpotOrdersResponse,
            "/injective.exchange.v1beta1.MsgBatchCancelDerivativeOrdersResponse":
                injective_exchange_tx_pb.MsgBatchCancelDerivativeOrdersResponse,
            "/injective.exchange.v1beta1.MsgBatchCreateSpotLimitOrdersResponse":
                injective_exchange_tx_pb.MsgBatchCreateSpotLimitOrdersResponse,
            "/injective.exchange.v1beta1.MsgBatchCreateDerivativeLimitOrdersResponse":
                injective_exchange_tx_pb.MsgBatchCreateDerivativeLimitOrdersResponse,
            "/injective.exchange.v1beta1.MsgBatchUpdateOrdersResponse":
                injective_exchange_tx_pb.MsgBatchUpdateOrdersResponse,
            "/injective.exchange.v1beta1.MsgDepositResponse":
                injective_exchange_tx_pb.MsgDepositResponse,
            "/injective.exchange.v1beta1.MsgWithdrawResponse":
                injective_exchange_tx_pb.MsgWithdrawResponse,
            "/injective.exchange.v1beta1.MsgSubaccountTransferResponse":
                injective_exchange_tx_pb.MsgSubaccountTransferResponse,
            "/injective.exchange.v1beta1.MsgLiquidatePositionResponse":
                injective_exchange_tx_pb.MsgLiquidatePositionResponse,
            "/injective.exchange.v1beta1.MsgIncreasePositionMarginResponse":
                injective_exchange_tx_pb.MsgIncreasePositionMarginResponse,
            "/injective.auction.v1beta1.MsgBidResponse":
                injective_auction_tx_pb.MsgBidResponse,
            "/injective.exchange.v1beta1.MsgCreateBinaryOptionsLimitOrderResponse":
                injective_exchange_tx_pb.MsgCreateBinaryOptionsLimitOrderResponse,
            "/injective.exchange.v1beta1.MsgCreateBinaryOptionsMarketOrderResponse":
                injective_exchange_tx_pb.MsgCreateBinaryOptionsMarketOrderResponse,
            "/injective.exchange.v1beta1.MsgCancelBinaryOptionsOrderResponse":
                injective_exchange_tx_pb.MsgCancelBinaryOptionsOrderResponse,
            "/injective.exchange.v1beta1.MsgAdminUpdateBinaryOptionsMarketResponse":
                injective_exchange_tx_pb.MsgAdminUpdateBinaryOptionsMarketResponse,
            "/injective.exchange.v1beta1.MsgInstantBinaryOptionsMarketLaunchResponse":
                injective_exchange_tx_pb.MsgInstantBinaryOptionsMarketLaunchResponse,
            "/cosmos.bank.v1beta1.MsgSendResponse":
                cosmos_bank_tx_pb.MsgSendResponse,
            "/cosmos.authz.v1beta1.MsgGrantResponse":
                cosmos_authz_tx_pb.MsgGrantResponse,
            "/cosmos.authz.v1beta1.MsgExecResponse":
                cosmos_authz_tx_pb.MsgExecResponse,
            "/cosmos.authz.v1beta1.MsgRevokeResponse":
                cosmos_authz_tx_pb.MsgRevokeResponse,
            "/injective.oracle.v1beta1.MsgRelayPriceFeedPriceResponse":
                injective_oracle_tx_pb.MsgRelayPriceFeedPriceResponse,
            "/injective.oracle.v1beta1.MsgRelayProviderPricesResponse":
                injective_oracle_tx_pb.MsgRelayProviderPrices,
        }
        # fmt: on
        msgs = []
        for msg in data.msg_responses:
            msgs.append(header_map[msg.type_url].FromString(msg.value))

        return msgs

    @staticmethod
    def UnpackMsgExecResponse(msg_type, data):
        # fmt: off
        header_map = {
            "MsgCreateSpotLimitOrder":
                injective_exchange_tx_pb.MsgCreateSpotLimitOrderResponse,
            "MsgCreateSpotMarketOrder":
                injective_exchange_tx_pb.MsgCreateSpotMarketOrderResponse,
            "MsgCreateDerivativeLimitOrder":
                injective_exchange_tx_pb.MsgCreateDerivativeLimitOrderResponse,
            "MsgCreateDerivativeMarketOrder":
                injective_exchange_tx_pb.MsgCreateDerivativeMarketOrderResponse,
            "MsgCancelSpotOrder":
                injective_exchange_tx_pb.MsgCancelSpotOrderResponse,
            "MsgCancelDerivativeOrder":
                injective_exchange_tx_pb.MsgCancelDerivativeOrderResponse,
            "MsgBatchCancelSpotOrders":
                injective_exchange_tx_pb.MsgBatchCancelSpotOrdersResponse,
            "MsgBatchCancelDerivativeOrders":
                injective_exchange_tx_pb.MsgBatchCancelDerivativeOrdersResponse,
            "MsgBatchCreateSpotLimitOrders":
                injective_exchange_tx_pb.MsgBatchCreateSpotLimitOrdersResponse,
            "MsgBatchCreateDerivativeLimitOrders":
                injective_exchange_tx_pb.MsgBatchCreateDerivativeLimitOrdersResponse,
            "MsgBatchUpdateOrders":
                injective_exchange_tx_pb.MsgBatchUpdateOrdersResponse,
            "MsgDeposit":
                injective_exchange_tx_pb.MsgDepositResponse,
            "MsgWithdraw":
                injective_exchange_tx_pb.MsgWithdrawResponse,
            "MsgSubaccountTransfer":
                injective_exchange_tx_pb.MsgSubaccountTransferResponse,
            "MsgLiquidatePosition":
                injective_exchange_tx_pb.MsgLiquidatePositionResponse,
            "MsgIncreasePositionMargin":
                injective_exchange_tx_pb.MsgIncreasePositionMarginResponse,
            "MsgCreateBinaryOptionsLimitOrder":
                injective_exchange_tx_pb.MsgCreateBinaryOptionsLimitOrderResponse,
            "MsgCreateBinaryOptionsMarketOrder":
                injective_exchange_tx_pb.MsgCreateBinaryOptionsMarketOrderResponse,
            "MsgCancelBinaryOptionsOrder":
                injective_exchange_tx_pb.MsgCancelBinaryOptionsOrderResponse,
            "MsgAdminUpdateBinaryOptionsMarket":
                injective_exchange_tx_pb.MsgAdminUpdateBinaryOptionsMarketResponse,
            "MsgInstantBinaryOptionsMarketLaunch":
                injective_exchange_tx_pb.MsgInstantBinaryOptionsMarketLaunchResponse,
        }
        # fmt: on

        responses = [header_map[msg_type].FromString(result) for result in data.results]
        return responses

    @staticmethod
    def UnpackTransactionMessages(transaction):
        meta_messages = json.loads(transaction.messages.decode())
        # fmt: off
        header_map = {
            "/injective.exchange.v1beta1.MsgCreateSpotLimitOrder":
                injective_exchange_tx_pb.MsgCreateSpotLimitOrder,
            "/injective.exchange.v1beta1.MsgCreateSpotMarketOrder":
                injective_exchange_tx_pb.MsgCreateSpotMarketOrder,
            "/injective.exchange.v1beta1.MsgCreateDerivativeLimitOrder":
                injective_exchange_tx_pb.MsgCreateDerivativeLimitOrder,
            "/injective.exchange.v1beta1.MsgCreateDerivativeMarketOrder":
                injective_exchange_tx_pb.MsgCreateDerivativeMarketOrder,
            "/injective.exchange.v1beta1.MsgCancelSpotOrder":
                injective_exchange_tx_pb.MsgCancelSpotOrder,
            "/injective.exchange.v1beta1.MsgCancelDerivativeOrder":
                injective_exchange_tx_pb.MsgCancelDerivativeOrder,
            "/injective.exchange.v1beta1.MsgBatchCancelSpotOrders":
                injective_exchange_tx_pb.MsgBatchCancelSpotOrders,
            "/injective.exchange.v1beta1.MsgBatchCancelDerivativeOrders":
                injective_exchange_tx_pb.MsgBatchCancelDerivativeOrders,
            "/injective.exchange.v1beta1.MsgBatchCreateSpotLimitOrders":
                injective_exchange_tx_pb.MsgBatchCreateSpotLimitOrders,
            "/injective.exchange.v1beta1.MsgBatchCreateDerivativeLimitOrders":
                injective_exchange_tx_pb.MsgBatchCreateDerivativeLimitOrders,
            "/injective.exchange.v1beta1.MsgBatchUpdateOrders":
                injective_exchange_tx_pb.MsgBatchUpdateOrders,
            "/injective.exchange.v1beta1.MsgDeposit":
                injective_exchange_tx_pb.MsgDeposit,
            "/injective.exchange.v1beta1.MsgWithdraw":
                injective_exchange_tx_pb.MsgWithdraw,
            "/injective.exchange.v1beta1.MsgSubaccountTransfer":
                injective_exchange_tx_pb.MsgSubaccountTransfer,
            "/injective.exchange.v1beta1.MsgLiquidatePosition":
                injective_exchange_tx_pb.MsgLiquidatePosition,
            "/injective.exchange.v1beta1.MsgIncreasePositionMargin":
                injective_exchange_tx_pb.MsgIncreasePositionMargin,
            "/injective.auction.v1beta1.MsgBid":
                injective_auction_tx_pb.MsgBid,
            "/injective.exchange.v1beta1.MsgCreateBinaryOptionsLimitOrder":
                injective_exchange_tx_pb.MsgCreateBinaryOptionsLimitOrder,
            "/injective.exchange.v1beta1.MsgCreateBinaryOptionsMarketOrder":
                injective_exchange_tx_pb.MsgCreateBinaryOptionsMarketOrder,
            "/injective.exchange.v1beta1.MsgCancelBinaryOptionsOrder":
                injective_exchange_tx_pb.MsgCancelBinaryOptionsOrder,
            "/injective.exchange.v1beta1.MsgAdminUpdateBinaryOptionsMarket":
                injective_exchange_tx_pb.MsgAdminUpdateBinaryOptionsMarket,
            "/injective.exchange.v1beta1.MsgInstantBinaryOptionsMarketLaunch":
                injective_exchange_tx_pb.MsgInstantBinaryOptionsMarketLaunch,
            "/cosmos.bank.v1beta1.MsgSend":
                cosmos_bank_tx_pb.MsgSend,
            "/cosmos.authz.v1beta1.MsgGrant":
                cosmos_authz_tx_pb.MsgGrant,
            "/cosmos.authz.v1beta1.MsgExec":
                cosmos_authz_tx_pb.MsgExec,
            "/cosmos.authz.v1beta1.MsgRevoke":
                cosmos_authz_tx_pb.MsgRevoke,
            "/injective.oracle.v1beta1.MsgRelayPriceFeedPrice":
                injective_oracle_tx_pb.MsgRelayPriceFeedPrice,
            "/injective.oracle.v1beta1.MsgRelayProviderPrices":
                injective_oracle_tx_pb.MsgRelayProviderPrices,
        }
        # fmt: on
        msgs = []
        for msg in meta_messages:
            msg_as_string_dict = json.dumps(msg["value"])
            msgs.append(json_format.Parse(msg_as_string_dict, header_map[msg["type"]]()))

        return msgs

    def _initialize_markets_and_tokens_from_files(self):
        config: ConfigParser = constant.CONFIGS[self.network]
        spot_markets = dict()
        derivative_markets = dict()
        tokens = dict()

        for section_name, configuration_section in config.items():
            if section_name.startswith("0x"):
                description = configuration_section["description"]

                quote_token = Token(
                    name="",
                    symbol="",
                    denom="",
                    address="",
                    decimals=int(configuration_section["quote"]),
                    logo="",
                    updated=-1,
                )

                if "Spot" in description:
                    base_token = Token(
                        name="",
                        symbol="",
                        denom="",
                        address="",
                        decimals=int(configuration_section["base"]),
                        logo="",
                        updated=-1,
                    )

                    market = SpotMarket(
                        id=section_name,
                        status="",
                        ticker=description,
                        base_token=base_token,
                        quote_token=quote_token,
                        maker_fee_rate=None,
                        taker_fee_rate=None,
                        service_provider_fee=None,
                        min_price_tick_size=Decimal(str(configuration_section["min_price_tick_size"])),
                        min_quantity_tick_size=Decimal(str(configuration_section["min_quantity_tick_size"])),
                    )
                    spot_markets[market.id] = market
                else:
                    market = DerivativeMarket(
                        id=section_name,
                        status="",
                        ticker=description,
                        oracle_base="",
                        oracle_quote="",
                        oracle_type="",
                        oracle_scale_factor=1,
                        initial_margin_ratio=None,
                        maintenance_margin_ratio=None,
                        quote_token=quote_token,
                        maker_fee_rate=None,
                        taker_fee_rate=None,
                        service_provider_fee=None,
                        min_price_tick_size=Decimal(str(configuration_section["min_price_tick_size"])),
                        min_quantity_tick_size=Decimal(str(configuration_section["min_quantity_tick_size"])),
                    )

                    derivative_markets[market.id] = market

            elif section_name != "DEFAULT":
                token = Token(
                    name=section_name,
                    symbol=section_name,
                    denom=configuration_section["peggy_denom"],
                    address="",
                    decimals=int(configuration_section["decimals"]),
                    logo="",
                    updated=-1,
                )

                tokens[token.symbol] = token

        self.tokens = tokens
        self.spot_markets = spot_markets
        self.derivative_markets = derivative_markets
        self.binary_option_markets = dict()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: injective/peggy/v1/params.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1finjective/peggy/v1/params.proto\x12\x12injective.peggy.v1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\"\xb7\x07\n\x06Params\x12\x10\n\x08peggy_id\x18\x01 \x01(\t\x12\x1c\n\x14\x63ontract_source_hash\x18\x02 \x01(\t\x12\x1f\n\x17\x62ridge_ethereum_address\x18\x03 \x01(\t\x12\x17\n\x0f\x62ridge_chain_id\x18\x04 \x01(\x04\x12\x1d\n\x15signed_valsets_window\x18\x05 \x01(\x04\x12\x1d\n\x15signed_batches_window\x18\x06 \x01(\x04\x12\x1c\n\x14signed_claims_window\x18\x07 \x01(\x04\x12\x1c\n\x14target_batch_timeout\x18\x08 \x01(\x04\x12\x1a\n\x12\x61verage_block_time\x18\t \x01(\x04\x12#\n\x1b\x61verage_ethereum_block_time\x18\n \x01(\x04\x12M\n\x15slash_fraction_valset\x18\x0b \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12L\n\x14slash_fraction_batch\x18\x0c \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12L\n\x14slash_fraction_claim\x18\r \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12X\n slash_fraction_conflicting_claim\x18\x0e \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12&\n\x1eunbond_slashing_valsets_window\x18\x0f \x01(\x04\x12X\n slash_fraction_bad_eth_signature\x18\x10 \x01(\x0c\x42.\xc8\xde\x1f\x00\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\x12\x19\n\x11\x63osmos_coin_denom\x18\x11 \x01(\t\x12\"\n\x1a\x63osmos_coin_erc20_contract\x18\x12 \x01(\t\x12\x1e\n\x16\x63laim_slashing_enabled\x18\x13 \x01(\x08\x12$\n\x1c\x62ridge_contract_start_height\x18\x14 \x01(\x04\x12\x36\n\rvalset_reward\x18\x15 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00:\x04\x80\xdc \x00\x42MZKgithub.com/InjectiveLabs/injective-core/injective-chain/modules/peggy/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'injective.peggy.v1.params_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZKgithub.com/InjectiveLabs/injective-core/injective-chain/modules/peggy/types'
  _PARAMS.fields_by_name['slash_fraction_valset']._options = None
  _PARAMS.fields_by_name['slash_fraction_valset']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_batch']._options = None
  _PARAMS.fields_by_name['slash_fraction_batch']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_claim']._options = None
  _PARAMS.fields_by_name['slash_fraction_claim']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_conflicting_claim']._options = None
  _PARAMS.fields_by_name['slash_fraction_conflicting_claim']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['slash_fraction_bad_eth_signature']._options = None
  _PARAMS.fields_by_name['slash_fraction_bad_eth_signature']._serialized_options = b'\310\336\037\000\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec'
  _PARAMS.fields_by_name['valset_reward']._options = None
  _PARAMS.fields_by_name['valset_reward']._serialized_options = b'\310\336\037\000'
  _PARAMS._options = None
  _PARAMS._serialized_options = b'\200\334 \000'
  _globals['_PARAMS']._serialized_start=110
  _globals['_PARAMS']._serialized_end=1061
# @@protoc_insertion_point(module_scope)
